//--------------------------------------------------------------------//
//
// Functions to provide Gnuplot <-> Plplot compatibility
// These functions are meant to serve as a simple tool so that
// functions/programs written to use the Plplot interface might
// also work with the Gnuplot interface. 
//
// Exact replication of Plplot functionality or appearance is 
// _not_ the goal, or the result. The plot may not appear the same
// as it would with the Plplot interface, and some functionality is
// missing do to certain fundamental behaviors of the two plot
// engines.
//

//
// Ian Searle, 7/6/97
//

//--------------------------------------------------------------------//

plstart = function ( NCOL, NROW, DEVICE )
{
  if (!exist (NROW)) { NROW = 1; }
  if (!exist (NCOL)) { NCOL = 1; }
  
  // Ignore the DEVICE argument.
  
  multiplot (NROW, NCOL);
};

//--------------------------------------------------------------------//

plmesh = function ( LIST )
{
  if (!exist (LIST.x)) {
    error ("plmesh: input LIST must contain `x' element");
  }

  if (!exist (LIST.y)) {
    error ("plmesh: input LIST must contain `y' element");
  }
  if (!exist (LIST.z)) {
    error ("plmesh: input LIST must contain `z' element");
  }

  splot ( LIST.x, LIST.y, LIST.z );
};

//--------------------------------------------------------------------//

plcont = function ( LIST )
{
  if (!exist (LIST.x)) {
    error ("plcont: input LIST must contain `x' element");
  }

  if (!exist (LIST.y)) {
    error ("plcont: input LIST must contain `y' element");
  }
  if (!exist (LIST.z)) {
    error ("plcont: input LIST must contain `z' element");
  }

  cont ( LIST.x, LIST.y, LIST.z );
};

//--------------------------------------------------------------------//

pltitle = function ( TITLE )
{
  title ( TITLE );
};

//--------------------------------------------------------------------//

plegend = function ( LEGEND )
{
  key ();
};

//--------------------------------------------------------------------//

plgrid = function ( GSTYX, GSTYY )
{
  // A no-op for now
  return 0;
};

//--------------------------------------------------------------------//

plstyle = function ( STYLE )
{
  if (!exist (STYLE)) 
  { 
    linestyle ( "lines" );
  }

  if (STYLE == "line")
  {
    linestyle ( "lines" );
  }

  if (STYLE == "point")
  {
    linestyle ( "points" );
  }

  if (STYLE == "line-point")
  {
    linestyle ( "linespoints" );
  }
};

//--------------------------------------------------------------------//

plaxis = function ( XSTR, YSTR )
{
  if (!exist (XSTR)) { XSTR = ""; }
  if (!exist (YSTR)) { YSTR = ""; }

  if (XSTR == "" && YSTR == "")
  {
    nolog ();
  }

  if (XSTR == "log" && YSTR == "log")
  {
    loglog ();
  }

  if (XSTR == "log" && YSTR == "")
  {
    semilogx ();
  }

  if (XSTR == "" && YSTR == "log")
  {
    semilogy ();
  }

};

//--------------------------------------------------------------------//

plimits = function ( Xmin, Xmax, Ymin, Ymax, Zmin, Zmax )
{
  if (exist (Xmin) && exist (Xmax))
  {
    xrange (Xmin, Xmax);
    else
    autoscale ("x");
  }

  if (exist (Ymin) && exist (Ymax))
  {
    yrange (Ymin, Umax);
    else
    autoscale ("y");
  }

  if (exist (Zmin) && exist (Zmax))
  {
    zrange (Zmin, Zmax);
    else
    autoscale ("z");
  }
};

//--------------------------------------------------------------------//

plaspect = function ( ASPR )
{
  if (!exist (ASPR))
  {
    plot ("set size noratio");
  else
    plot ("set size ratio " + num2str (ASPR));
  }
};

//--------------------------------------------------------------------//

subplot = function ( N )
{
  // Can't do anything here...
  return 1;
};

//--------------------------------------------------------------------//

plclose = function ( I )
{
  pclose ( I );
};

//--------------------------------------------------------------------//

plwid = function ( WID )
{
  // Do nothing...
  return 0;
};

//--------------------------------------------------------------------//

plfont = function ( FONT )
{
  // Do nothing...
  return 0;
};

//--------------------------------------------------------------------//

plptex = function ( TEXT, X, Y, Z )
{
  // Do nothing for now...
  return 0;
};

//--------------------------------------------------------------------//

plaz = function ( AZ )
{
  pview (, AZ);
};

//--------------------------------------------------------------------//

plalt = function ( ALT )
{
  // Do nothing ?
  return 0;
};

//--------------------------------------------------------------------//
