//-------------------------------------------------------------------//

//  Synopsis:   Create a "set" vector.

//  Syntax:	set ( A )

//  Description:

//  The set function takes an vector argument (A), and returns a
//  vector that is sorted in ascending order, and has no duplicate
//  values. 

//  See Also: complement, intersection, union

//-------------------------------------------------------------------//

set = function ( A )
{
  if (min (size (A)) != 1) 
  {
    error ("set: requires vector argument");
  }

  i = j = 1;

  # Put the array in order.
  a = sort (A).val;

  # Now, remove any duplicates
  while (i <= a.n)
  {
    tmp = find (a[i] == a);
    if (tmp.n > 1) 
    {
      # Found more than one instance of a number.
      s[j] = a[tmp[1]];
      i = i + tmp.n;
    else
      # Only found the one instance.
      s[j] = a[tmp];
      i++; 
    }
    j++;
  }

  return s
};
