//---------------------------------------------------------------------

//  Synopsis:   Convolve two vectors.

//  Syntax:     conv ( A , B )

//  Description:

//  This routine convolves the vectors a and b. This function uses a
//  Finite Impulse Response filter to do the convolution. This method
//  can be faster than using the DFT when the sizes of A and B are
//  different. 

//---------------------------------------------------------------------

conv = function( A , B )
{
  na = max (size (A));
  nb = max (size (B));

  if (na > nb)
  {
    if (nb > 1) {
      A[na+nb-1] = 0;
    }
    c = filter (B, 1, A);
  else
    if (na > 1) {
      B[na+nb-1] = 0;
    }
    c = filter (A, 1, B);
  }

  return c.y;
};
