//-------------------------------------------------------------------//

//  Syntax:     phase ( A )

//  Description:

//  Calculate the phase, or the angle, of a complex vector. Normally
//  we are tempted to do this with a simple atan2() function
//  call. However, this method often results in discontinuities at the
//  pi boundaries. The method used herein attempts to keep the phase
//  continuous over the pi boundaries.

//-------------------------------------------------------------------//

phase = function ( A )
{
  global(pi)

  # Force a column vector  A = A[:];
  phi = atan2 (imag(A), real(A));
  N = phi.n;

  # Compute the difference between each element of phi...
  dphi = phi[1:N-1] - phi[2:N];

  # Now, find the indices where dphi is greater than pi.
  I = find (abs (dphi) > 3.5);

  # Now step through the indices, compensating for the pi boundary...
  for (i in I)
  {
    if (i != 0)
    {
      phi = phi + 2*pi*sign(dphi[i])*[zeros(1,i), ones(1,N-i)];
    }
  }

  return phi;
};
